VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "ApiEnumeration"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

'==================================================================
'
' API Enumeration Class
'
' 29 September 2001
'
' You're free in using this software, as long as the
' orignal copyright/author information remains intact.
' In other words, respecting the person(s) who made it
' possible for you to view/learn something of this software.
'
' Copyright  2001 - Diederik van der Boor
' http://www.codingdomain.f2s.com
'
'==================================================================


'------------------------------------------------------------------
' Class Usage:
'
' Private WithEvents APIEnum As APIEnumeration
'
' Sub Button_Click()
'   Set APIEnum = New APIEnumaration
'   APIEnum.EnumAPIs("C:\Windows\System\shell32.dll")
' End Sub
'
' Private Sub APIEnum_GotAPI(Name As String, ContinueEnum As Boolean)
'   ...
' End Sub
'
'------------------------------------------------------------------

'------------------------------------------------------------------
' This event is fired for every API found in the DLL

Public Event GotAPI(Name As String, ContinueEnum As Boolean)



'------------------------------------------------------------------
' Constants for API reading in DLL's

Private Const INVALID_HANDLE_VALUE = -1
Private Const OPEN_EXISTING = 3

Private Enum FileAccess
  GENERIC_READ = &H80000000
  GENERIC_WRITE = &H40000000
End Enum
  
Private Enum FileShare
  FILE_SHARE_READ = &H1
  FILE_SHARE_WRITE = &H2
End Enum

Private Enum FileAttributes ' Some aren't the same as the vb versions.
  FILE_ATTRIBUTE_ARCHIVE = &H20
  FILE_ATTRIBUTE_COMPRESSED = &H800
  FILE_ATTRIBUTE_DIRECTORY = &H10
  FILE_ATTRIBUTE_HIDDEN = &H2
  FILE_ATTRIBUTE_NORMAL = &H80
  FILE_ATTRIBUTE_READONLY = &H1
  FILE_ATTRIBUTE_SYSTEM = &H4
  FILE_ATTRIBUTE_TEMPORARY = &H100
End Enum

Private Type IMAGE_DEBUG_INFORMATION
  List As Long
  Size As Long
  MappedBase As Long
  Machine As Long
  Characteristics As Long
  CheckSum As Long
  ImageBase As Long
  SizeOfImage As Long
  NumberOfSections As Long
  Sections As Long                       'pointer to a IMAGE_SECTION_HEADER type
  ExportedNamesSize As Long
  ExportedNames As Long
  NumberOfFunctionTableEntries As Long
  FunctionTableEntries As Long           'pointer to a IMAGE_FUNCTION_ENTRY type
  LowestFunctionStartingAddress As Long
  HighestFunctionEndingAddress As Long
  NumberOfFpoTableEntries As Long
  FpoTableEntries As Long
  SizeOfCoffSymbols As Long
  CoffSymbols As Long                    'pointer to the COFF symbol table
  SizeOfCodeViewSymbols As Long
  CodeViewSymbols As Long
  ImageFilePath As Long
  ImageFileName As Long
  DebugFilePath As Long
  TimeDateStamp As Long
  RomImage As Long
  DebugDirectory As Long                 'pointer to a IMAGE_DEBUG_DIRECTORY type
  NumberOfDebugDirectories As Long
  OriginalFunctionTableBaseAddress As Long
  Reserved0 As Long
  Reserved1 As Long
  Reserved2 As Long
End Type

Private Declare Function MapDebugInformation Lib "ImageHlp.dll" (ByVal FileHandle As Long, ByVal FileName As String, ByVal SymbolPath As String, ByVal ImageBase As Long) As Long
Private Declare Function UnmapDebugInformation Lib "ImageHlp.dll" (ByVal DebugInfo As Long) As Long

Private Declare Function CreateFile Lib "kernel32" Alias "CreateFileA" (ByVal lpFileName As String, ByVal dwDesiredAccess As FileAccess, ByVal dwShareMode As FileShare, ByVal lpSecurityAttributes As Long, ByVal dwCreationDisposition As Long, ByVal dwFlagsAndAttributes As FileAttributes, ByVal hTemplateFile As Long) As Long
Private Declare Function CloseHandle Lib "kernel32" (ByVal hObject As Long) As Long

Private Declare Sub CopyMemory Lib "kernel32" Alias "RtlMoveMemory" (pDest As Any, pSource As Any, ByVal ByteLen As Long)




'------------------------------------------------------------------
' Call this method to start the enumeration

Public Sub EnumAPIs(DLLName As String)
  Dim ImgDbg As IMAGE_DEBUG_INFORMATION
  Dim hFile As Long
  Dim PtrDbg As Long
  Dim Result As String
  Dim Pos As Long
  Dim ContinueEnum As Boolean
  
  'obtain a handle to the specified file
  hFile = CreateFile(DLLName, GENERIC_READ, FILE_SHARE_READ Or FILE_SHARE_WRITE, 0&, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0&)
  If hFile = INVALID_HANDLE_VALUE Then Err.Raise 12345, "File APIs", "The DLL file was invalid!"
  
  'obtain a pointer to the image debug info in the passed file
  PtrDbg = MapDebugInformation(hFile, DLLName, vbNullString, 0&)
  
  'fill the structure with the image debug info
  Call CopyMemory(ImgDbg, ByVal PtrDbg, Len(ImgDbg))
  
  'pad a string sufficiently large enough to
  'receive the string of exported functions
  'and copy the functions into the string.
  With ImgDbg
    Result = String(.ExportedNamesSize, vbNullChar)
    Call CopyMemory(ByVal Result, ByVal .ExportedNames, .ExportedNamesSize)
  End With
  
  Call UnmapDebugInformation(PtrDbg)
  Call CloseHandle(hFile)
  
  ' RTrim null chars
  Do While Right(Result, 1) = vbNullChar
    Result = Left(Result, Len(Result) - 1)
  Loop
  
  ' Find Items
  ContinueEnum = True
  Do
    Pos = InStr(1, Result, vbNullChar)
    If Pos Then
      RaiseEvent GotAPI(Left(Result, Pos - 1), ContinueEnum)
      Result = Mid(Result, Pos + 1)
    End If
  Loop While Pos <> 0 And ContinueEnum
End Sub
